@@ -20,19 +20,21 @@ from django.conf import settings
 from django.core import signals
 from django.core.exceptions import ImproperlyConfigured
 from django.dispatch import receiver
-from django.middleware.csrf import get_token
+from django.middleware import csrf
 from django.template import RequestContext
 from django.template import TemplateDoesNotExist
 from django.template import TemplateSyntaxError
 from django.template.backends.base import BaseEngine
 from django.template.backends.utils import csrf_input_lazy
 from django.template.backends.utils import csrf_token_lazy
+from django.template.context import BaseContext
 from django.utils import lru_cache
 from django.utils import six
 from django.utils.encoding import smart_text
 from django.utils.functional import SimpleLazyObject
 from django.utils.functional import cached_property
 from django.utils.module_loading import import_string
+from django.utils.safestring import mark_safe
 
 from . import base
 from . import builtins
@@ -54,21 +56,21 @@ class Template(object):
     def __init__(self, template, backend):
         self.template = template
         self.backend = backend
-
-        self._debug = False
         self.origin = Origin(name=template.filename, template_name=template.name)
 
     @property
     def name(self):
-        return self.origin.name
+        return self.template.name
 
     def render(self, context=None, request=None):
         if context is None:
             context = {}
 
+        context = base.dict_from_context(context)
+
         if request is not None:
             def _get_val():
-                token = get_token(request)
+                token = csrf.get_token(request)
                 if token is None:
                     return 'NOTPROVIDED'
                 else:
@@ -83,7 +85,6 @@ class Template(object):
 
         if self.backend._tmpl_debug:
             from django.test import signals
-            from django.template.context import BaseContext
 
             # Define a "django" like context for emitatet the multi
             # layered context object. This is mainly for apps like
@@ -102,7 +103,7 @@ class Template(object):
                                            context=context)
 
 
-        return self.template.render(context)
+        return mark_safe(self.template.render(context))
 
 
 class Jinja2(BaseEngine):
@@ -172,7 +173,14 @@ class Jinja2(BaseEngine):
 
         environment_cls = import_string(environment_clspath)
 
-        options.setdefault("loader", jinja2.FileSystemLoader(self.template_dirs))
+        if isinstance(options.get("loader"), six.string_types):
+            # Allow to specify a loader as string
+            loader_cls = import_string(options.pop("loader"))
+        else:
+            # Backward compatible default
+            loader_cls = jinja2.FileSystemLoader
+
+        options.setdefault("loader", loader_cls(self.template_dirs))
         options.setdefault("extensions", builtins.DEFAULT_EXTENSIONS)
         options.setdefault("auto_reload", settings.DEBUG)
         options.setdefault("autoescape", True)
